# multikit "invoked command" for SDP-specified "video" programs
# It is a stub for an helper program that has a different argument signature
# (otherwise we could have just invoked the helper program directly)
# version: 1999.01.07

# The main routine (invoked command):

proc mk_sdp_video {ourProgId ourInternetAddress ourPortNum ourTTL ourPublicKey args} {
	array set attributeArgs $args
	set medium video

	set sessionName "SDP $medium session"
	catch {set sessionName $attributeArgs(nickname)}

	# protocol specification
	set protocol rtp
	catch {set protocol [string tolower $attributeArgs(sdp_transport)]}
	if {$protocol == "rtp" || $protocol == "rtp/avp"} {
		set rtpFlag "-r"
	} else {
		set rtpFlag ""
	}

	# format specification
	set formats ""
	catch {set formats $attributeArgs(sdp_formats)}
	# I'm not sure how we're supposed to handle more than one format:
	if {[llength $formats] > 1} {
		puts "mk_sdp_$medium warning: multiple formats \"$formats\" for session \"$sessionName\""
	}
	set format [lindex $formats 0]
	# convert known numeric formats to string names used by "vic":
	switch -exact $format {
		25	{set format celb}
		28	{set format nv}
		31	{set format h261}
	}
	if {$format == "ivs"} {
		# special case:
		set formatFlag "-f h261 -Aivs"
	} elseif {$format != ""} {
		set formatFlag "-f $format"
	} else {
		set formatFlag ""
	}
	
	# misc attributes
	set sdp_attributes ""
	catch {set sdp_attributes $attributeArgs(sdp_session_attributes)}
	catch {set sdp_attributes [concat $sdp_attributes $attributeArgs(sdp_media_attributes)]}

	set execCmd "vic $formatFlag -t $ourTTL -C [list $sessionName] $ourInternetAddress/$ourPortNum"
	if {![catch {eval exec $execCmd &} pid]} {
		noteSpawnedProcess $ourInternetAddress $ourPortNum $ourTTL $pid
		return
	}

	tk_messageBox -title "mk_sdp_$medium: error" \
		-message "Failed to exec \"$execCmd\": $pid" \
		-icon warning -type ok
}

proc noteSpawnedProcess {addr port ttl pid} {
	# Now that we've successfully started a background process to implement
	# the invoked command, we inform multikit of the group/port(s) that
	# we're using (in case they need to be tunneled):
      set groupsocks [multikit_createRTPgroupsocks $addr $port $ttl]
	foreach gs $groupsocks {
		groupsock_turnOnBackgroundReadHandling $gs
	}

	# Finally, periodically check for the death of the spawned process:
	checkSpawnedProcess $pid $groupsocks
}

proc checkSpawnedProcess {pid groupsocks} {
	if {![processExists $pid]} {
		eval multikit_deleteRTPgroupsocks $groupsocks
	} else {
		after 10000 [list checkSpawnedProcess $pid $groupsocks]
	}
}
